<?php

namespace HS\Domain\Documents\VirusScan;

use Illuminate\Http\Client\ConnectionException;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;

class VirusScanner
{
    /**
     * @var string
     */
    protected $apiKey;

    /**
     * @param $apiKey
     */
    public function __construct($apiKey)
    {
        $this->apiKey = $apiKey;
    }

    public function scanContent($content)
    {
        return $this->doScan(md5($content));
    }

    public function scanFile($filePath)
    {
        if (file_exists($filePath)) {
            return $this->doScan(md5_file($filePath));
        }

        Log::error(new \Exception('Could not scan file, temp file not found '.$filePath));

        // Trust, if we cannot determine status
        return ScanResult::clean();
    }

    public function doScan($md5)
    {
        try {
            $response = Http::timeout(2)
                ->withHeaders(['x-apikey' => $this->apiKey])
                ->get('https://www.virustotal.com/api/v3/search', [
                    'query' => $md5,
                ]);

            if ($response->successful()) {
                return ($this->determineIsClean($response->json('data')))
                    ? ScanResult::clean()
                    : ScanResult::suspicious();
            } else {
                Log::error(new \Exception('Bad request to virustotal'), [
                    'status' => $response->status(),
                    'message' => $response->body(),
                ]);
                // falls down to callback at end of function
            }
        } catch(ConnectionException $e) {
            // Trust, if we get a timeout
            Log::error($e);
            return ScanResult::clean();
        }

        return ScanResult::clean();
    }

    protected function determineIsClean($data)
    {
        return (
            empty($data[0]['attributes']['popular_threat_classification'] ?? []) // Has no threat classification
            &&
            ($data[0]['attributes']['last_analysis_stats']['malicious'] ?? 0 ) == 0 // Analysis shows not malicious
        );
    }
}
